using Microsoft.Extensions.DependencyInjection;
using Volo.Abp.AspNetCore.Mvc.UI.Bundling;
using Volo.Abp.AspNetCore.Mvc.UI.MultiTenancy;
using Volo.Abp.AspNetCore.Mvc.UI.Theme.Commercial;
using Ravco.Abp.AspNetCore.Mvc.UI.Theme.Front.Bundling;
using Ravco.Abp.AspNetCore.Mvc.UI.Theme.Front.ObjectMapping;
using Ravco.Abp.AspNetCore.Mvc.UI.Theme.Front.Themes.Front.Libraries.Aos;
using Ravco.Abp.AspNetCore.Mvc.UI.Theme.Front.Themes.Front.Libraries.Bootstrap;
using Ravco.Abp.AspNetCore.Mvc.UI.Theme.Front.Themes.Front.Libraries.BootstrapIcons;
using Ravco.Abp.AspNetCore.Mvc.UI.Theme.Front.Themes.Front.Libraries.FSLightbox;
using Ravco.Abp.AspNetCore.Mvc.UI.Theme.Front.Themes.Front.Libraries.HS.HSGoTo;
using Ravco.Abp.AspNetCore.Mvc.UI.Theme.Front.Themes.Front.Libraries.HS.HSHeader;
using Ravco.Abp.AspNetCore.Mvc.UI.Theme.Front.Themes.Front.Libraries.HS.HSMegaMenu;
using Ravco.Abp.AspNetCore.Mvc.UI.Theme.Front.Themes.Front.Libraries.HS.HSShowAnimation;
using Ravco.Abp.AspNetCore.Mvc.UI.Theme.Front.Themes.Front.Libraries.TypedJs;
using Volo.Abp.AspNetCore.Mvc.UI.Theme.Shared;
using Volo.Abp.AspNetCore.Mvc.UI.Theming;
using Volo.Abp.AutoMapper;
using Ravco.Abp.FrontTheme.Management;
using Volo.Abp;
using Volo.Abp.AspNetCore.Mvc.UI.Packages.FlagIconCss;
using Volo.Abp.Modularity;
using Volo.Abp.VirtualFileSystem;

namespace Ravco.Abp.AspNetCore.Mvc.UI.Theme.Front;

[DependsOn(
    typeof(AbpAspNetCoreMvcUiMultiTenancyModule),
    typeof(AbpAutoMapperModule),
    typeof(AbpAspNetCoreMvcUiThemeCommercialModule),
    typeof(FrontThemeManagementDomainSharedModule)
    )]
public class AbpAspNetCoreMvcUiFrontThemeModule : AbpModule
{
    public override void PreConfigureServices(ServiceConfigurationContext context)
    {
        PreConfigure<IMvcBuilder>(mvcBuilder =>
        {
            mvcBuilder.AddApplicationPartIfNotExists(typeof(AbpAspNetCoreMvcUiFrontThemeModule).Assembly);
        });
    }

    public override void ConfigureServices(ServiceConfigurationContext context)
    {
        Configure<AbpThemingOptions>(options =>
        {
            options.Themes.Add<FrontTheme>();

            if (options.DefaultThemeName == null)
            {
                options.DefaultThemeName = FrontTheme.Name;
            }
        });

        Configure<AbpErrorPageOptions>(options =>
        {
            options.ErrorViewUrls.Add("401", "~/Views/Error/401.cshtml");
            options.ErrorViewUrls.Add("403", "~/Views/Error/403.cshtml");
            options.ErrorViewUrls.Add("404", "~/Views/Error/404.cshtml");
            options.ErrorViewUrls.Add("500", "~/Views/Error/500.cshtml");
        });

        Configure<AbpVirtualFileSystemOptions>(options =>
        {
            options.FileSets.AddEmbedded<AbpAspNetCoreMvcUiFrontThemeModule>();
        });

        Configure<AbpBundlingOptions>(options =>
        {
                options
                    .ScriptBundles
                    .Add(FrontThemeBundles.Scripts.Global, bundle =>
                    {
                        bundle
                            .AddContributors(typeof(FrontGlobalScriptContributor));
                    })
                    .Add(FrontThemeBundles.Scripts.Layouts.Empty, bundle =>
                    {
                        bundle
                            .AddContributors(typeof(FrontHsHeaderScriptContributor))
                            .AddContributors(typeof(FrontHsGoToScriptContributor))
                            .AddContributors(typeof(FrontHsMegaMenuScriptContributor));
                    })
                    .Add(FrontThemeBundles.Scripts.Layouts.Account, bundle =>
                    {
                        bundle
                            .AddContributors(typeof(FrontBootstrapScriptContributorExtension))
                            .AddFiles("/Themes/Front/Global/scripts/account-custom.js");
                    })                    
                    .Add(FrontThemeBundles.Scripts.Layouts.Public, bundle =>
                    {
                        bundle
                            .AddContributors(typeof(FrontHsHeaderScriptContributor))
                            .AddContributors(typeof(FrontHsMegaMenuScriptContributor))
                            .AddContributors(typeof(FrontHsShowAnimationScriptContributor))
                            .AddContributors(typeof(FrontHsGoToScriptContributor))
                            .AddContributors(typeof(FrontAosScriptContributor))
                            .AddContributors(typeof(FrontTypedJsScriptContributor))
                            .AddContributors(typeof(FrontFSLightboxScriptContributor))
                            .AddContributors(typeof(FrontBootstrapScriptContributorExtension))
                            .AddFiles("/Themes/Front/Global/scripts/public-custom.js");
                    });

                options
                    .StyleBundles
                    .Add(FrontThemeBundles.Styles.Global, bundle =>
                    {
                        bundle
                            .AddContributors(typeof(FrontBootstrapIconsStyleContributor))
                            .AddContributors(typeof(FrontGlobalStyleContributor));
                    })
                    .Add(FrontThemeBundles.Styles.Layouts.Empty, bundle =>
                    {
                        bundle
                            .AddContributors(typeof(FlagIconCssStyleContributor))
                            .AddContributors(typeof(FrontHsMegaMenuStyleContributor));
                    })
                    .Add(FrontThemeBundles.Styles.Layouts.Account, bundle =>
                    {
                        bundle
                            .AddContributors(typeof(FrontBootstrapIconsStyleContributor));
                    })
                    .Add(FrontThemeBundles.Styles.Layouts.Public, bundle =>
                    {
                        bundle
                            .AddContributors(typeof(FrontHsMegaMenuStyleContributor))
                            .AddContributors(typeof(FrontAosStyleContributor));
                    });
        });

        context.Services.AddAutoMapperObjectMapper<AbpAspNetCoreMvcUiFrontThemeModule>();

        Configure<AbpAutoMapperOptions>(options =>
        {
            options.AddProfile<FrontThemeAutoMapperProfile>(validate: true);
        });
    }

    public override void OnApplicationInitialization(ApplicationInitializationContext context)
    {
        
    }
}
