using System;
using System.Threading.Tasks;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Options;
using Ravco.Abp.FrontTheme.Management;
using Volo.Abp.AspNetCore.Mvc.UI.Bundling;
using Volo.Abp.AspNetCore.Mvc.UI.Theme.Shared.Bundling;
using Volo.Abp.Localization;
using Volo.Abp.Modularity;
using Volo.Abp.Settings;

namespace Ravco.Abp.AspNetCore.Mvc.UI.Theme.Front.Bundling;

[DependsOn(
    typeof(SharedThemeGlobalStyleContributor)
)]
public class FrontGlobalStyleContributor : BundleContributor
{
    public override async Task ConfigureBundleAsync(BundleConfigurationContext context)
    {
        var options = context.ServiceProvider.GetRequiredService<IOptions<FrontThemeOptions>>().Value;

        if (!options.StylePath.IsNullOrEmpty())
        {
            context.Files.Add(options.StylePath);
            return;
        }

        var cssFile = await GetStyleCssFileNameAsync(context, options);
        context.Files.Add($"/Themes/Front/Global/Styles/{cssFile}");

        context.Files.RemoveAll(x => x == (CultureHelper.IsRtl
            ? "/libs/bootstrap/css/bootstrap.rtl.css"
            : "/libs/bootstrap/css/bootstrap.css"));
    }

    private static async Task<string> GetStyleCssFileNameAsync(
        BundleConfigurationContext context,
        FrontThemeOptions options)
    {
        var styleSettingName = options.IsDashboardWebsite
            ? FrontThemeSettingNames.ApplicationLayoutStyle
            : FrontThemeSettingNames.PublicLayoutStyle;

        var style = await context.ServiceProvider
            .GetRequiredService<ISettingProvider>()
            .GetOrNullAsync(styleSettingName);

        if (string.Equals(style, FrontStyle.Style1.ToString(), StringComparison.OrdinalIgnoreCase))
        {
            return "front1.css";
        }
        
        if (string.Equals(style, FrontStyle.Style2.ToString(), StringComparison.OrdinalIgnoreCase))
        {
            return "front2.css";
        }

        return "front1.css";
    }
}
